#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "restart-io.hh"

template <class ProgramState>
RestartIO<ProgramState>::RestartIO(HDF5::Grouplike &file, size_t vertexCount)
    : group_(file, "restarts"),
      displacementWriter_(group_, "displacement", vertexCount,
                          Vector::block_type::dimension),
      velocityWriter_(group_, "velocity", vertexCount,
                      Vector::block_type::dimension),
      accelerationWriter_(group_, "acceleration", vertexCount,
                          Vector::block_type::dimension),
      stateWriter_(group_, "state", vertexCount),
      weightedNormalStressWriter_(group_, "weightedNormalStress", vertexCount),
      relativeTimeWriter_(group_, "relativeTime"),
      relativeTimeIncrementWriter_(group_, "relativeTimeIncrement") {}

template <class ProgramState>
void RestartIO<ProgramState>::write(ProgramState const &programState) {
  addEntry(displacementWriter_, programState.timeStep, programState.u);
  addEntry(velocityWriter_, programState.timeStep, programState.v);
  addEntry(accelerationWriter_, programState.timeStep, programState.u);
  addEntry(stateWriter_, programState.timeStep, programState.alpha);
  addEntry(weightedNormalStressWriter_, programState.timeStep,
           programState.weightedNormalStress);
  addEntry(relativeTimeWriter_, programState.timeStep,
           programState.relativeTime);
  addEntry(relativeTimeIncrementWriter_, programState.timeStep,
           programState.relativeTau);
}

template <class ProgramState>
void RestartIO<ProgramState>::read(size_t timeStep,
                                   ProgramState &programState) {
  programState.timeStep = timeStep;
  readEntry(displacementWriter_, timeStep, programState.u);
  readEntry(velocityWriter_, timeStep, programState.v);
  readEntry(accelerationWriter_, timeStep, programState.u);
  readEntry(stateWriter_, timeStep, programState.alpha);
  readEntry(weightedNormalStressWriter_, timeStep,
            programState.weightedNormalStress);
  readEntry(relativeTimeWriter_, timeStep, programState.relativeTime);
  readEntry(relativeTimeIncrementWriter_, timeStep, programState.relativeTau);
}

#include "restart-io_tmpl.cc"
